/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>

#include <artnet/artnet.h>
#include <artnet/packets.h>

#define MAX_ERRORS 5

int8_t buff[ARTNET_MAX_DMX +1] ;
int length ;
char *dev = "/dev/dmx0" ;
int verbose = 0;

void do_write(int fd) {
	int res ;
	
	res = write(fd, buff, length+1);

	if(verbose)
		printf("0x%02hhx, 0x%02hhx, 0x%02hhx, 0x%02hhx length %d\n", buff[1], buff[2], buff[3], buff[4] , length) ;
	
	if (res < 0){
		/* seems like if you unplug devices from the dongle
		 * it causes a write error, in this case we try and reopen 
		 * the device and write the data again, else give up
		 */
		perror("Error writing to device");
		
		res = close(fd) ;
		if(res < 0) 
			perror("close ") ;

		fd = open(dev,O_WRONLY);
	
		if (fd < 0) {
			perror("open");
			exit(-1);
		}
		res = write(fd, buff, length+1);

		if(res < 0) {
			printf("giving up now\n") ;
			exit(-1);
		}
	} 
}

int dmx_callback(artnet_node n, int port, void *d) {
	int8_t *data ;
	int *fd = (int*) d ;

	data = artnet_read_dmx(n, port, &length) ;
	memset(buff, 0x00, ARTNET_MAX_DMX+1) ;
	memcpy(&buff[1], data, length) ;

	do_write(*fd) ;
	return 0;
}



int main(int argc, char *argv[]) {
	artnet_node node ;
	int fd ;
	char *ip_addr = NULL ;
	int optc, subnet_addr = 0, port_addr = 0 ;

	// parse options 
	while ((optc = getopt (argc, argv, "s:p:d:a:v")) != EOF) {
		switch (optc) {
 			case 'a':
				ip_addr = (char *) strdup(optarg) ;
		        break;
			case 'v':
				verbose = 1 ;
		        break;
			case 'd':
				dev = (char *) strdup(optarg) ;
		        break;
			case 's':
				subnet_addr = atoi(optarg) ;

				if(subnet_addr < 0 && subnet_addr > 15) {
					printf("Subnet address must be between 0 and 15\n") ;
					exit(1) ;
				}
				break ;
			case 'p':
				port_addr = atoi(optarg) ;

				if(port_addr < 0 && port_addr > 15) {
					printf("Port address must be between 0 and 15\n") ;
					exit(1) ;
				}
				break ;
      		default:
				break;
    	}
	}

    node = artnet_init(ip_addr, verbose) ; ;

	artnet_set_short_name(node, "Artnet -> DMX ") ;
	artnet_set_long_name(node, "ArtNet to DMX convertor") ;
	artnet_set_node_type(node, ARTNET_NODE) ;
	artnet_set_dmx_handler(node, dmx_callback, (void*) &fd) ;

	// set the first port to output dmx data
	artnet_set_port_type(node, 0, ARTNET_ENABLE_OUTPUT, ARTNET_PORT_DMX) ;
	artnet_set_subnet_addr(node, subnet_addr) ;

	// set the universe address of the first port
	artnet_set_port_addr(node, 0, ARTNET_OUTPUT_PORT, port_addr) ;
	artnet_start(node) ;

	fd = open(dev,O_WRONLY);
	if (fd < 0) {
		perror("open");
		exit(-1);
	}

	while(1) {
		// set a 1 second timeout on the read
		// this way we send a DMX frame every second
		// even if we don't get any ArtNet packets
		artnet_read(node, 1) ;
		do_write(fd) ;
	}

	return 0 ;	
}
